
var timeStart;
var enabled = 1;

var init = 0;

/**
 * Start the timer as soon as the script is loaded.
 */
$(document).ready(function() {
  initialize_timer();
});

/**
 * Display a warning if user navigates away from page without saving timer
 * information
 */
$(document).ready(function() {
  if (Drupal.settings.support_timer.unload_warning) {
    $('#comment-form').bind("change", function() { confirm_unload(true); });
    $('#node-form').bind("change", function() { confirm_unload(true); });
    $("#edit-submit").click(function() { window.confirm_unload(false); });
    $("#edit-preview").click(function() { window.confirm_unload(false); });
  }
});

/**
 * Toggle the timer (on pause/resume).
 */
function pause_timer() {
  // If timer is running, pause it.
  if (enabled) {
    // Update the displayed value.
    update_timer();
    enabled = 0;
    // Replace "Pause" with "Resume".
    jQuery("#edit-pause")[0].value = "Resume";
  }
  // Else, timer is already paused, resume.
  else {
    enabled = 1;
    // Reload timer from entry field (in case it was manually edited).
    read_time_field();
    // Replace "Resume" with "Pause".
    jQuery("#edit-pause")[0].value = "Pause";
  }
}

/**
 * Reset the timer to 00:00:00.
 */
function reset_timer() {
  start_timer(0);
  // Push value to entry field so update happens even if the timer is paused.
  jQuery("#edit-elapsed").val(ms_to_hhmmss(0));
  if (!enabled) {
    // Replace "Pause" with "Start".
    jQuery("#edit-pause")[0].value = "Start";
  }
}

/**
 * Read system clock when timer starts.
 */
function start_timer(initial_value) {
  timeStart = new Date().getTime() - initial_value;
  update_timer();
}

/**
 * Update the edit-elapsed textfield with the current elapsed time.
 */
function update_timer() {
  if (enabled) {
    // Calculated elapsed time.
    var elapsed = new Date().getTime() - timeStart;
    // Update the displayed value.
    jQuery("#edit-elapsed").val(ms_to_hhmmss(elapsed));
    // Set timeout to return here in approximately one second.
    window.setTimeout(function() {
      update_timer();
    }, 1000);
  }
}

/**
 * Read the current contents of the elapsed time field and update the timer.
 */
function read_time_field() {
  hhmmss = jQuery("#edit-elapsed").val();
  ms = (hhmmss == undefined) ? 0 : hhmmss_to_ms(hhmmss);
  start_timer(ms);
}

/**
 * Initialize the timer on page-load.
 */
function initialize_timer() {
  enabled = Drupal.settings.support_timer.autostart;
  if (!enabled) {
    // Replace "Pause" with "Start".
    jQuery("#edit-pause")[0].value = "Start";
  }
  read_time_field();
}

/**
 * Determine whether or not we should display a message when a user navigates
 * away from the current page.
 */
function confirm_unload(on) {
  window.onbeforeunload = (on) ? unload_message : null;
}

/**
 * The message we display when a user navigates away from a changed ticket
 * without saving his/her changes.
 */
function unload_message() {
  return Drupal.t('Any timer information you have entered for this support ticket will be lost if you navigate away from this page without pressing the Save button.');
}

/**
 * Convert time string hh:mm:ss to numeric, in milliseconds.
 */
function hhmmss_to_ms(elapsed) {
  var hms = elapsed.split(':');

  // Return zero if format is invalid.
  if (hms.length != 3) {
    return 0;
  }

  // Get time in seconds.
  var ss = 0;
  for (i = 0; i < 3; i++) {
    ss = ss * 60 + parseInt(hms[i], 10);
  }

  // Return time in milliseconds.
  return 1000 * ss;
}

/**
 * Convert milliseconds to string formatted as hh:mm:ss.
 */
function ms_to_hhmmss(ms) {
  var hh, mm, ss;

  // Convert time in ms to hours, minutes and seconds.
  ss = Math.round(ms / 1000);
  hh = Math.floor(ss / 3600);
  ss -= hh * 3600;
  mm = Math.floor(ss / 60);
  ss -= mm * 60;

  // Add leading zero where needed.
  if (hh < 10) {
    hh = '0' + hh;
  }
  if (mm < 10) {
    mm = '0' + mm;
  }
  if (ss < 10) {
    ss = '0' + ss;
  }

  // Return time formatted as hh:mm:ss.
  return hh + ':' + mm + ':' + ss;
}
